<?php

/*
 * Window message plugin.
 * Displays all messages like new driven records and system messages into a window
 * instead within the chat.
 * ----------------------------------------------------------------------------------
 * Author:	undef.de
 * Version:	0.9.6
 * Date:	2009-12-20
 * Home:	http://www.undef.de/Trackmania/Plugins/
 * System:	xaseco/1.06 and up
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 * Do not use this plugin in combination with "plugin.msglog.php"!
 */

/* The following manialink id's are used in this plugin (the 910 part of id can be changed on trouble):
 * 91000	id for manialinks
 * 91001	id for manialink MessageWindow
 */

Aseco::registerEvent('onStartup',			'winmsg_onStartup');
Aseco::registerEvent('onStatusChangeTo3',		'winmsg_setActive');
Aseco::registerEvent('onPlayerConnect',			'winmsg_onPlayerConnect');
Aseco::registerEvent('onPlayerDisconnect',		'winmsg_onPlayerDisconnect');
Aseco::registerEvent('onEndRound',			'winmsg_setInactive');

Aseco::addChatCommand('togglemsgwin',			'Toggle the display of the message window');

global $winmsg_config, $WINMSG_VERSION, $WINMSGS;


$winmsg_config['WINDOW_POS_X'] = -39.75;		// Change to place the window to an other location
$winmsg_config['WINDOW_POS_Y'] = 47.85;			// Change to place the window to an other location
$winmsg_config['HIDE_TM_NOTICE_MESSAGES'] = true;	// Hide all "Player XYZ connected to the game..." and "Player XYZ takes the 1. place..." above left
$winmsg_config['TEXTSHADOW'] = false;			// Shadow behind the text? 'true' = Shadow on; 'false' = Shadow off
$winmsg_config['TEXTSIZE'] = 1;				// 1, 2 or 3 (1 = default)
$winmsg_config['SCALE'] = 1;				// 0.1 up to 1 (= 100%), e.g. 0.2 or 0.5 ...
$winmsg_config['AUTOHIDE'] = 0;				// Set to '7' to show message window 7 sec., then hide complete, 0 for always on.
$winmsg_config['AMOUNT_MESSAGES'] = 9;			// Max. 9 Messages
$winmsg_config['OPACITY'] = array('1','3','5','8','A','B','D','E','F');


// DO NOT CHANGE THIS!!!
$winmsg_config['STATUS'] = true;
$winmsg_config['MANIALINK_ID'] = '910';

$WINMSG_VERSION = '0.9.6';
$WINMSGS = array();

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onStartup
function winmsg_onStartup ($aseco, $command) {
	global $winmsg_config;

	if ( version_compare(XASECO_VERSION, 1.06, '<') ) {
		trigger_error('[window.message.php] Not supported xaseco version ('. XASECO_VERSION .')! Please update to min. version 1.06!', E_USER_ERROR);
	}

	if ($winmsg_config['SCALE'] == 0) {
		$winmsg_config['SCALE'] = 1;
	}
	if ($winmsg_config['SCALE'] > 1) {
		$winmsg_config['SCALE'] = 1;
	}

	if ($winmsg_config['TEXTSIZE'] <= 0) {
		$winmsg_config['TEXTSIZE'] = 1;
	}
	if ($winmsg_config['TEXTSIZE'] > 3) {
		$winmsg_config['TEXTSIZE'] = 1;
	}

	if ($winmsg_config['AUTOHIDE'] > 0) {
		$winmsg_config['AUTOHIDE'] = $winmsg_config['AUTOHIDE'] * 1000;
	}

	// Set multiplier for line-height of Textsizes
	$winmsg_config['MULTIPLIER'] = array(2,2,5,9);

	if ($winmsg_config['HIDE_TM_NOTICE_MESSAGES'] == true) {
		setCustomUIField('notice', false);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerConnect
function winmsg_onPlayerConnect ($aseco, $player) {
	global $winmsg_config, $WINMSG_VERSION, $WINMSGS;

	// Init the Player Array
	$WINMSGS[$player->login] = array();

	$player->data['WindowMessageStatus'] = 'visible';

	$message = '$0B3This is the plugin {#highlite}$l[http://www.undef.de/Trackmania/Plugins/]window.message.php/'. $WINMSG_VERSION .'$l$0B3 for xaseco.'. LF .'$0B3To hide this message type {#highlite}/togglemsgwin';
	send_window_message($aseco, $message, $player);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerDisconnect
function winmsg_onPlayerDisconnect ($aseco, $player) {
	global $WINMSGS;

	// Remove Player log from array to save mem
	$tmp = array();
	foreach ($WINMSGS as $ply => $array) {
		if ($ply != $player->login) {
			$tmp[$ply] = $array;
		}
	}
	$WINMSGS = $tmp;
	unset($tmp);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onNewChallenge
function winmsg_setActive ($aseco) {
	global $winmsg_config;

	$winmsg_config['STATUS'] = true;

	// display the messages
	foreach ($aseco->server->players->player_list as $player) {
		winmsg_displayMessageWindow($aseco, $player);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEndRace, onEndRound
function winmsg_setInactive ($aseco) {
	global $winmsg_config;

	$winmsg_config['STATUS'] = false;

	// display the messages
	foreach ($aseco->server->players->player_list as $player) {
		winmsg_hideMessageWindow($aseco, $player);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Save the messages into an array
// $param can be 'true', 'false' or the player array
function send_window_message ($aseco, $message, $param = false) {
	global $winmsg_config, $WINMSGS;

	// If $param is be the player array, send the message to the given player
	if ( ($param !== true) && ($param !== false) && ($param->id) ) {

		// Add message to Player
		winmsg_addMessageToPlayer($aseco, $param, $message);

		if ($winmsg_config['STATUS'] == true) {
			winmsg_displayMessageWindow($aseco, $param);
		}
	}
	else {
		// Display the messages to all players
		foreach ($aseco->server->players->player_list as $player) {

			// Add message to Player
			winmsg_addMessageToPlayer($aseco, $player, $message);

			if ($winmsg_config['STATUS'] == true) {
				winmsg_displayMessageWindow($aseco, $player);
			}
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Add an new message to the Player Array
function winmsg_addMessageToPlayer ($aseco, $player, $message) {
	global $winmsg_config, $WINMSGS;

	// Transform CRLF to LF
	$message = str_replace("\r", '', $message);

	// Add the new message
	$msg = split(LF, $message);
	if ( is_array($msg) ) {
		foreach ($msg as &$text) {
			$text = winmsg_cleanUp($text);
		}
		$WINMSGS[$player->login] = array_merge($WINMSGS[$player->login], $msg);
	}
	else {
		array_push($WINMSGS[$player->login], winmsg_cleanUp($msg));
	}
	unset($msg);


	// If more then $winmsg_config['AMOUNT_MESSAGES'] messages in the array, get the last $winmsg_config['AMOUNT_MESSAGES']
	if ( count($WINMSGS[$player->login]) > $winmsg_config['AMOUNT_MESSAGES']) {
		$WINMSGS[$player->login] = array_slice($WINMSGS[$player->login], (count($WINMSGS[$player->login]) - $winmsg_config['AMOUNT_MESSAGES']), $winmsg_config['AMOUNT_MESSAGES']);
	}


	// If less then $winmsg_config['AMOUNT_MESSAGES'], just fill up
	if ( count($WINMSGS[$player->login]) < $winmsg_config['AMOUNT_MESSAGES']) {
		$tmp = array();
		for ($i = count($WINMSGS[$player->login]); $i < $winmsg_config['AMOUNT_MESSAGES']; $i++) {
			array_push($tmp, ' ');
		}
		$WINMSGS[$player->login] = array_merge($tmp, $WINMSGS[$player->login]);
		unset($tmp);
	}

	// If message window is hidden for this player, so just show all messages in chat too.
	if ($player->data['WindowMessageStatus'] == 'hidden') {
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $player->login);
	}

}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function winmsg_cleanUp ($message) {
	// clean up for xml
	// http://en.tm-wiki.org/wiki/Text_Formatting
	$message = preg_replace('/^(\{#[A-Z]+\}|\$[A-F0-9TISWNMGZOHLP\$]+)(>|>>) (.*)$/i', '$1$3', $message);	// remove > or >> from message or all colors and formating codes
	$message = str_ireplace('$S', '', $message);								// remove $s case-insensitive
	$message = str_replace('"', '&quot;', $message);
	$message = str_replace("'", '&apos;', $message);
	$message = str_replace('<', '&lt;', $message);
	$message = str_replace('>', '&gt;', $message);

	return $message;
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// displays the message window
function winmsg_displayMessageWindow ($aseco, $player) {
	global $winmsg_config, $WINMSGS;

	// When the player wish to hide this window, bail out.
	if ($player->data['WindowMessageStatus'] == 'hidden') {
		return;
	}

	$content =  '<?xml version="1.0" encoding="UTF-8"?>';
	$content .= '<manialinks id="'. $winmsg_config['MANIALINK_ID'] .'00">';
	$content .= '<manialink id="'. $winmsg_config['MANIALINK_ID'] .'01">';
	$content .= '<frame posn="'. $winmsg_config['WINDOW_POS_X'] .' '. $winmsg_config['WINDOW_POS_Y'] .' 0">';
	$line = 0;
	$opacity = 0;
	foreach ($WINMSGS[$player->login] as $msg) {
		if ($winmsg_config['TEXTSHADOW'] == true) {
			$msg = '$S'. $msg;
		}
		$content .= '<label sizen="79.6 0" posn="0 '. -$line .' 0" scale="'. $winmsg_config['SCALE'] .'" textsize="'. $winmsg_config['TEXTSIZE'] .'" textcolor="FFF'. $winmsg_config['OPACITY'][$opacity] .'" text="'. $aseco->formatColors($msg) .'"/>';
		$line += (($winmsg_config['SCALE'] / $winmsg_config['TEXTSIZE']) * $winmsg_config['MULTIPLIER'][$winmsg_config['TEXTSIZE']]);
		$opacity += 1;
	}
	$content .= '</frame>';
	$content .= '</manialink>';
	$content .= '</manialinks>';

	$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, $content, $winmsg_config['AUTOHIDE'], false));
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEndRace
function winmsg_hideMessageWindow ($aseco, $player) {
	global $winmsg_config, $WINMSGS;

	// If the Window is already hidden, bail out.
	if ($player->data['WindowMessageStatus'] == 'hidden') {
		return;
	}

	$content =  '<?xml version="1.0" encoding="UTF-8"?>';
	$content .= '<manialinks id="'. $winmsg_config['MANIALINK_ID'] .'00">';
	$content .= '<manialink id="'. $winmsg_config['MANIALINK_ID'] .'01">';
	$content .= '</manialink>';
	$content .= '</manialinks>';

	$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, $content, 0, false));
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function chat_togglemsgwin ($aseco, $command) {

	if ($command['author']->data['WindowMessageStatus'] == 'hidden') {
		$command['author']->data['WindowMessageStatus'] = 'visible';
		winmsg_displayMessageWindow($aseco, $command['author']);

		$message = '{#server}> $0F3Your message window is now shown, type {#highlite}/togglemsgwin$0F3 to hide them.';
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
	}
	else {
		winmsg_hideMessageWindow($aseco, $command['author']);
		$command['author']->data['WindowMessageStatus'] = 'hidden';

		$message = '{#server}> $0F3Your message window is now hidden, type {#highlite}/togglemsgwin$0F3 to reactivate them.';
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
	}
}

?>
